#!

$script = <<'EOD';
~startperl~ -w
use lib "~libdir~";
use Autofont::PSres;

# CMR and LaTeX fonts
my(%cmrfonts, $cmrfont);
foreach $cmrfont (qw(CMB10 CMBSY10 CMBX10 CMBX12 CMBX5 CMBX6 CMBX7 CMBX8
		     CMBX9 CMBXSL10 CMBXTI10 CMCSC10 CMDUNH10 CMEX10 CMFF10
		     CMFI10 CMFIB8 CMINCH CMITT10 CMMI10 CMMI12 CMMI5 CMMI6
		     CMMI7 CMMI8 CMMI9 CMMIB10 CMR10 CMR12 CMR17 CMR5 CMR6
		     CMR7 CMR8 CMR9 CMSL10 CMSL12 CMSL8 CMSL9 CMSLTT10
		     CMSS10 CMSS12 CMSS17 CMSS8 CMSS9 CMSSBX10 CMSSDC10
		     CMSSI10 CMSSI12 CMSSI17 CMSSI8 CMSSI9 CMSSQ8 CMSSQI8
		     CMSY10 CMSY5 CMSY6 CMSY7 CMSY8 CMSY9 CMTCSC10 CMTEX10
		     CMTEX8 CMTEX9 CMTI10 CMTI12 CMTI7 CMTI8 CMTI9 CMTT10
		     CMTT12 CMTT8 CMTT9 CMU10 CMVTT10 LASY10 LASY5 LASY6
		     LASY7 LASY8 LASY9 LASYB10 LCIRCLE10 LCIRCLEW10 LCMSS8
		     LCMSSB8 LCMSSI8 LINE10 LINEW10 LOGO10 LOGO8 LOGO9
		     LOGOBF10 LOGOSL10 LINEW10 CMBSY5 CMBSY7 CMMIB5 CMMIB7
		     EUEX10 EUFB10 EUFB5 EUFB7 EUFM10 EUFM5 EUFM7 EURB10
		     EURB5 EURB7 EURM10 EURM5 EURM7 EUSB10 EUSB5 EUSB7
		     EUSM10 EUSM5 EUSM7 MSAM10 MSAM5 MSAM7 MSBM10 MSBM5
		     MSBM7 WNCYB10 WNCYI10 WNCYR10 WNCYSC10 WNCYSS10)) {
    $cmrfonts{$cmrfont} = 1;
}

# generic subroutines
my($exitvalue, $anyexitvalue) = (0, 0);
sub mysystem ($) {
    my($command) = @_;
    print STDERR "-- $command\n";
    $exitvalue = system($command) / 256;
    $anyexitvalue = $exitvalue if $exitvalue;
}


# environment variables

$_ = `kpsewhich --show-path='type1 fonts'`; chomp;
my(@type1path) = split(/:(?!!!)|:?!!/);
my($PSres) = new Autofont::PSres(@type1path);

$pkdestdir = $ENV{'PKDESTDIR'} || '/usr/tmp/pk';
if (!-d $pkdestdir) {
    mkdir($pkdestdir, 0777) || die "can't make destination directory";
}

$ps2pk = $ENV{'PS2PK'} || 'ps2pk';
$gsftopk = $ENV{'GSFTOPK'} || 'gsftopk';


##
# Arguments

my($mfmode, $resolution, $baseresolution, $magsteps, $oldstyle);
$oldstyle = 1;

while ($ARGV[0] =~ /^-/) {
    $_ = shift @ARGV;
    if (/^--mfmode$/) {
	$mfmode = shift @ARGV;
	$oldstyle = 0;
    } elsif (/^--bdpi$/) {
	$baseresolution = shift @ARGV;
	$oldstyle = 0;
    } elsif (/^--mag$/) {
	$magsteps = shift @ARGV;
	$oldstyle = 0;
    } elsif (/^--dpi$/) {
	$resolution = shift @ARGV;
	$oldstyle = 0;
    } elsif (/^-v$/) {
	$ps2pk .= ' -v';
    } else {
	die "Usage: not $_\n";
    }
}

$want_pk_name = shift @ARGV;
if ($oldstyle) {
    ($resolution, $baseresolution, $magsteps, $mfmode) = @ARGV;
}

my($output_file) = "$pkdestdir/$want_pk_name.${resolution}pk";

# parse dvips configuration files
$psheaderpath = `kpsewhich --expand-path '\$TEXPSHEADERS'`;
chomp $psheaderpath;
@psfontsmap = ('psfonts.map');

sub read_dvips_config ($) {
    my($fn) = @_;
    open(CFG, $fn) || die "$fn: $!\n";
    while (<CFG>) {
	if (/^H\s+(\S+)/) {
	    my($hp) = $1;
	    if ($hp =~ /(^|.*?:)($|:.*)/) {
		$psheaderpath = $1 . $psheaderpath . $2;
	    } else {
		$psheaderpath = $hp;
	    }
	} elsif (/^p\s+\+(\S+)/) {
	    push @psfontsmap, $1;
	} elsif (/^p\s+(\S+)/) {
	    @psfontsmap = ($1);
	}
    }
    close CFG;
}

sub find_file_on_path ($@) {
    my($fn, @path) = @_;
    my($dir, $slashes, %used);
    return $fn if $fn =~ m{^(./|../|/)};
    foreach $dir (@path) {
	$dir =~ s|^~/|$ENV{'HOME'}/|;
	next if $used{$dir} || $dir eq '';
	$used{$dir} = 1;
	($dir, $slashes) = ($dir =~ m|^(.*?)(/*)$|);
	if (length($slashes) >= 2
	    && $dir ne '.' && $dir ne '..'
	    && (stat($dir))[3] > 2) {
	    # recursive subdirectory searching
	    my(@subdir, $entry);
	    opendir(DIR, $dir eq '' ? '/' : $dir) || next;
	    while (defined(($entry = readdir(DIR)))) {
		return "$dir/$entry" if $entry eq $fn;
		push(@subdir, "$dir/$entry//") if -d "$dir/$entry" && $entry ne '.' && $entry ne '..';
	    }
	    closedir(DIR);
	    if (@subdir) {
		$entry = &find_file_on_path($fn, @subdir);
		return $entry if defined($entry);
	    }
	} elsif (-r "$dir/$fn") {
	    return "$dir/$fn";
	}
    }
    undef;
}

# read config.ps file
$_ = `kpsewhich --expand-path '\$TEXCONFIG'`; chomp;
@dvipsconfigpath = split(/:/);
$_ = `kpsewhich --expand-path '\$TEXFONTMAPS'`; chomp;
@fontmappath = split(/:/);
@fontmappath = @dvipsconfigpath if $_ !~ /\S/;

read_dvips_config($f)
    if defined(($f = find_file_on_path('config.ps', @dvipsconfigpath)));
read_dvips_config("$ENV{'HOME'}/.dvipsrc")
    if exists $ENV{'HOME'} && -r "$ENV{'HOME'}/.dvipsrc";
@psheaderpath = split(/:/, $psheaderpath);

# find the map line
$mapline = undef;
die if !defined $want_pk_name;
PSFONTSMAP: foreach $f (@psfontsmap) {
    next if !defined(($f = find_file_on_path($f, @fontmappath)));
    open(PSFONTS, $f) || next;
    while (<PSFONTS>) {
	$mapline = $_, last if (/^$::want_pk_name\s/);
    }
    last if defined($mapline);
}

# run another mktexpk if no map line found
sub new_mktexpk_arg ($$) {
    my($aname, $aval) = @_;
    defined($aval) ? " $aname '$aval'" : "";
}

sub run_other_mktexpk () {
    my($ok, $p) = 0;
    foreach $p (split /:/, $ENV{'PATH'}) {
	if ($ok && -x "$p/mktexpk") {
	    exec "$p/mktexpk"
		. new_mktexpk_arg('--dpi', $resolution)
		. new_mktexpk_arg('--bdpi', $baseresolution)
		. new_mktexpk_arg('--mag', $magsteps)
		. new_mktexpk_arg('--mfmode', $mfmode)
		. " $want_pk_name";
	} elsif ($ok && -x "$p/MakeTeXPK") {
	    exec "$p/MakeTeXPK $want_pk_name $resolution $baseresolution '$magsteps'";
	}
	$ok = 1 if substr($0, 0, length($p) + 1) eq "$p/";
    }
    die "can't find another mktexpk to run";
}

run_other_mktexpk if !defined($mapline);

# otherwise, found a mapline; parse it into components
my($original_mapline) = $mapline;
my(@words, @fonts, @encodings, @other_downloads, $postscript);
$postscript = '';
while (1) {
    if ($mapline =~ /^\s*<\[\s*(\S*)(.*)$/) {
	push @encodings, $1 if $1 ne '';
	$mapline = $2;
    } elsif ($mapline =~ /^\s*<<?\s*(\S*)(.*)$/) {
	my($what);
	if (lc(substr($1, -4)) eq '.enc') {
	    $what = \@encodings;
	} elsif (lc(substr($1, -4)) eq '.pfa' || lc(substr($1, -4)) eq '.pfb') {
	    $what = \@fonts;
	} else {
	    $what = \@other_downloads;
	}
	push @$what, $1 if $1 ne '';
	$mapline = $2;
    } elsif ($mapline =~ /^\s*\"([^\"]*)($|\".*)/) {
	$postscript .= " " . $1;
	$mapline = $2;
	$mapline =~ s/^\"//;
    } elsif ($mapline =~ /^\s*(\S+)(.*)/) {
	push @words, $1;
	$mapline = $2;
    } else {
	last;
    }
}

# check the $postscript component for special effects
my($ps2pk_ok) = 1;
my($slant, $extend, $encode);
while ($postscript !~ /^\s*$/) {
    if ($postscript =~ /^\s*([\d.eE+-]+)\s+SlantFont\b(.*)/) {
	$slant = (defined($slant) ? $slant + $1 : $1);
	$postscript = $2;
    } elsif ($postscript =~ /^\s*([\d.eE+-]+)\s+ExtendFont\b(.*)/) {
	$extend = (defined($extend) ? $extend + $1 : $1);
	$postscript = $2;
    } elsif ($postscript =~ /^\s*(\S+)\s+ReEncodeFont\b(.*)/) {
	$encode = $1;
	$postscript = $2;
    } else {
	$ps2pk_ok = 0;
    }
}

# load font definition
run_other_mktexpk if @words == 0;
push @words, $words[0] if @words == 1;
my($psname) = $words[1];
run_other_mktexpk if $cmrfonts{$psname};
my($multiple_master) = '';
if ($psname =~ /^([^_]*)(_[0-9._]+)$/) {
    ($psname, $multiple_master) = ($1, $2);
}
my($outlinefile) = $PSres->{'FontOutline'}->{$psname};
my($afmfile) = $PSres->{'FontAFM'}->{$psname};

# check for outline file in texmf tree
if (!$outlinefile && @fonts == 1) {
    $outlinefile = `kpsewhich --format='type1 fonts' $fonts[0]`;
    chomp $outlinefile;
}

# try to run ps2pk
$ps2pk = 0 if $multiple_master || !$outlinefile || !$afmfile
    || !-r $outlinefile || !-r $afmfile;
while ($ps2pk) {
    my($directory, $outlinename) = ($outlinefile =~ m|^(.*/)([^/]*)$|);
    my($afmdirectory) = ($afmfile =~ m|^(.*/)[^/]*$|);
    if ($afmdirectory ne $directory) {
	push @tempfiles, "/tmp/#$$#.afm", "/tmp/#$$#.pfb";
	system("cp $outlinefile /tmp/#$$#.pfb");
	system("cp $afmfile /tmp/#$$#.afm");
	$directory = "/tmp";
	$outlinename = "/tmp/#$$#.pfb";
    }

    my($ps2pk_args) = "";
    $ps2pk_args .= " -S$slant" if $slant;
    $ps2pk_args .= " -E$expand" if $expand;
    if ($encode) {
	$ps2pk = 0, last if @encodings != 1;
	$encode = find_file_on_path($encodings[0], @psheaderpath);
	$ps2pk = 0, last if !defined($encode);
	my($tempenc) = "/tmp/#$$#.enc";
	symlink($encode, $tempenc);
	$ps2pk_args .= " -e$tempenc";
	push(@tempfiles, $tempenc);
    }

    my($temppk) = "/tmp/#$$#.pk";
    chdir $directory || die "can't chdir";
    mysystem("$ps2pk$ps2pk_args -X$resolution -Y$resolution $outlinename $temppk >/dev/null");
    if (-r $temppk) {
	mysystem("mv $temppk $output_file");
    } else {
	$ps2pk = 0;
    }
    last;
}

# run gsftopk if you couldn't run ps2pk
if (!$ps2pk) {
    $anyexitvalue = 0;		# in case we tried ps2pk earlier
    chdir($pkdestdir) || die "can't chdir";

    if ($multiple_master) {
	push @tempfiles, "/tmp/#$$#.mmx";
	open OUT, ">/tmp/#$$#.mmx" || die;
	print OUT "/$psname findfont pop\n";
	close OUT;
	$original_mapline = "$original_mapline <[/tmp/#$$#.mmx";
    }

    chomp $original_mapline;
    $original_mapline =~ s/'/'"'"'/g;
    mysystem("$gsftopk --mapline='$original_mapline' $want_pk_name $resolution 1>&2");
}

# done
foreach $temp (@tempfiles) {
    unlink($temp);
}
exit if $anyexitvalue;

print "$output_file\n";

__END__

=head1 NAME

autofont-mktexpk - create a PK file for a font (Autofont version)

=head1 SYNOPSIS

autofont-mktexpk [I<options>] <font>

=head1 DESCRIPTION

B<autofont-mktexpk> generates a PK file at a given resolution for a given
font. The Autofont version is designed for PostScript Type 1 fonts, and
runs B<ps2pk> or B<gsftopk> to generate PK bitmaps. It searches your
L<dvips(1)> configuration files, the same way that B<dvips> does, searching
for a line in some F<psfonts.map> file that matches the provided font name.
It parses that line, then runs L<ps2pk(1)> and/or L<gsftopk(1)> to create
the requested PK file.

B<autofont-mktexpk> takes the same arguments as L<mktexpk(1)>, and like
that program, it is designed to be called automatically by L<xdvi(1)> and
related programs. To facilitate this, you should probably install a
symbolic link in your path so that `B<mktexpk>' actually runs
B<autofont-mktexpk>. Leave the original B<mktexpk> later in your path;
B<autofont-mktexpk> will run it for Metafonts and other fonts it cannot
process itself.

B<autofont-mktexpk> depends on PostScript F<PSres.upr> files generated by
L<autofont-mkpsres(1)> or L<makepsres(1)>.

=head1 ENVIRONMENT VARIABLES

B<autofont-mktexpk> uses environment variables to locate resources and to
determine where to place its generated output. In particular:

=over 8

=item F<PSres.upr> files

Searches PSRESOURCEPATH, T1FONTS, T1INPUTS, TEXPSHEADERS, and DVIPSHEADERS.

=item DVIPS configuration files

Searches TEXCONFIG, which defaults to F</usr/[local/]share/texmf/dvips> and
its subdirectories, for the first file named F<config.ps>; then reads
F<~/.dvipsrc> if it exists.

=item Font and encoding files

Searches DVIPSHEADERS or, if not specified, the directories specified in
the DVIPS configuration files.

=item Other B<mktexpk> scripts

Searches PATH.

=item PK output files

Places in PKDESTDIR, which defaults to F</tmp/pk>.

=item B<ps2pk> and B<gsftopk> programs

Uses PS2PK and GSFTOPK, which default to C<ps2pk> and C<gsftopk>,
respectively.

=head1 BUGS

Report bugs and send patches and enhancement requests to
C<ekohler@gmail.com>.

=head1 SEE ALSO

L<mktexpk(1)>, L<xdvi(1)>, L<ps2pk(1)>, L<gsftopk(1)>,
L<autofont-mkpsres(1)>, L<autofont-mktextfm(1)>

=head1 AUTHOR

Eddie Kohler, C<ekohler@gmail.com>

See L<http://www.lcdf.org/type/> for updates.

EOD

require Config;
$cfg{'startperl'} = $Config::Config{'startperl'};
$cfg{'libdir'} = `make libdir.txt >/dev/null 2>&1 && cat libdir.txt`;
chomp $cfg{'libdir'};

$script =~ s/\~(\w+)\~/$cfg{$1}/eg;

if (!open(FILE, ">autofont-mktexpk") || !print FILE ($script) || !close(FILE)) {
    die "Error while writing script: $!";
}
