#!

$script = <<'EOD';
~startperl~ -w
use lib "~libdir~";
use Autofont;
use Autofont::PSres;
use Autofont::VPL;
use Fcntl ':flock';

%fontmap =
    ('ptmr' => 'TimesRom',	'ptmri' => 'TimesIta',
     'ptmb' => 'TimesBol',	'ptmbi' => 'TimesBolIta',
     'phvr' => 'Helve',		'phvb' => 'HelveBol',
     'phvro' => 'HelveObl',	'phvbo' => 'HelveBolObl',
     'pcrr' => 'Couri',		'pcrb' => 'CouriBol',
     'pcrro' => 'CouriObl',	'pcrbo' => 'CouriBolObl');

%no_download_outline =
    ('Times-Roman' => 1,	'Times-Italic' => 1,
     'Times-Bold' => 1,		'Times-BoldItalic' => 1,
     'Helvetica' => 1,		'Helvetica-Oblique' => 1,
     'Helvetica-Bold' => 1,	'Helvetica-BoldOblique' => 1,
     'Courier' => 1,		'Courier-Oblique' => 1,
     'Courier-Bold' => 1,	'Courier-BoldOblique' => 1,
     'Symbol' => 1);

my($tmpdir, $keep_tmpdir);
my(@real_fonts);


##
# SUBROUTINES

# generic subroutines

my($exitvalue, $anyexitvalue) = (0, 0);

sub mysystem ($;$) {
    my($command, $in_tmpdir) = @_;
    print STDERR "-- $command\n";
    $command = "cd $tmpdir && $command" if $in_tmpdir;
    $exitvalue = system($command) / 256;
    $anyexitvalue = $exitvalue if $exitvalue;
}

sub sighandler {
    my $signame = shift;
    system("/bin/rm -rf $tmpdir") if $tmpdir && !$keep_tmpdir;
    exit(1);
}

END {
    system("/bin/rm -rf $tmpdir") if $tmpdir && !$keep_tmpdir;
}


# vpl creation subroutines

# make_vpl returns undef if it fails; otherwise, 1.

sub update_lsr ($) {
    my($fn) = @_;
    if ($fn && $writable_texmf
	&& substr($fn, 0, length($writable_texmf) + 1) eq "$writable_texmf/"
	&& -w "$writable_texmf/ls-R") {
	$fn = "./" . substr($fn, length($writable_texmf) + 1);
	my($dir, $file) = ($fn =~ m|^(.*)/([^/]*)$|);
	open(FOO, ">>$writable_texmf/ls-R") || die;
	print FOO "$dir:\n$file\n";
	close FOO;
    }
}

sub make_vpl ($) {
    my($font) = @_;

    # analyze the font; fail otherwise
    if (!$font->psname($PSres) || !$font->{'metrics_file'}) {
	return undef;
    }

    # set $font->{'basename'}, the name under which we will install TFMs
    $font->{'basename'} = $font->{'mmname'} . $font->{'suffix'}
    	if !$font->{'basename'};

    # move AFM to temporary directory; make AFM from any multiple-master
    # font if necessary
    my($metrics_path) = $font->{'metrics_file'};
    my($local_afm) = ($metrics_path =~ m|^.*/([^/]*)$|);
    if ($font->{'mm'}) {
	$local_afm =~ s/\.am?fm$//i;
	my($args) = '';
	foreach $i (1 .. @{$font->{'mmv'}}) {
	    my($val) = $font->{'mmv'}->[$i - 1];
	    $args .= " --$i=$val";
	    $local_afm .= "_$val";
	}
	$local_afm .= "_.afm";
	mysystem("$mmafm $metrics_path $args > $local_afm", 1);
	exit $exitvalue if $exitvalue != 0;
    } elsif (!-r "$tmpdir/$local_afm") {
	link($metrics_path, "$tmpdir/$local_afm")
	    || symlink($metrics_path, "$tmpdir/$local_afm")
		|| die;
    }

    # run afm2tfm
    my($args) = '';
    $args .= "-e $font->{'expand'} " if $font->{'expand'};
    $args .= "-s $font->{'slant'} " if $font->{'slant'};
    $args .= "-T " . $font->encoding()->filename() . " " if $font->encoding()->filename();
    $args .= ($font->{'fake_sc'} ? "-V " : "-v ");

    my($vpl_filename) = $local_afm;
    $vpl_filename =~ s/\.afm//i;
    $vpl_filename .= ".vpl";

    $font->{'vpl_file'} = "$tmpdir/$vpl_filename";
    $font->{'raw_tfm_file'} = "$tfm_dest_dir/$font->{'basename'}${Autofont::suffix_separator}0.tfm";
    mysystem("$afm2tfm $local_afm $args $vpl_filename $font->{'raw_tfm_file'} > /dev/null", 1);
    update_lsr($font->{'raw_tfm_file'});
    exit $exitvalue if $exitvalue != 0;

    push @real_fonts, $font;
    1;
}

sub generate_tfm ($) {
    my($font) = @_;
    return 0 if !$font->{'built'} && !make_vpl($font);
    mysystem("$vptovf $font->{'vpl_file'} $vf_dest_dir/$font->{'basename'}.vf $tfm_dest_dir/$font->{'basename'}.tfm > /dev/null");
    exit $exitvalue if $exitvalue != 0;
    update_lsr("$vf_dest_dir/$font->{'basename'}.vf");
    update_lsr("$tfm_dest_dir/$font->{'basename'}.tfm");
    if ($font->{'want_raw'}) {
	link $font->{'raw_tfm_file'}, "$tfm_dest_dir/$font->{'basename'}.tfm" || die;
    }
    1;
}


##
# Set up path and environment variables

$SIG{'INT'} = $SIG{'TERM'} = \&sighandler;

$_ = `kpsewhich --show-path='type1 fonts'`; chomp;
my(@type1path) = split(/:(?!!!)|:?!!/);
$PSres = new Autofont::PSres(@type1path);
$_ = `kpsewhich --show-path='PostScript header'`; chomp;
my(@psheaderpath) = split(/:(?!!!)|:?!!/);

$mmafm = $ENV{'MMAFM'} || 'mmafm';
$afm2tfm = $ENV{'AFM2TFM'} || 'afm2tfm';
$vptovf = $ENV{'VPTOVF'} || 'vptovf';

if ($ARGV[0] eq '-k') {
    $keep_tmpdir = 1;
    shift @ARGV;
}
$global_tmpdir = $ENV{'TMPDIR'} || '/tmp';
$global_tmpdir =~ s/\/$//;
$tmpdir = "$global_tmpdir/\%autofont" . ($keep_tmpdir ? '' : ".$$");
if (!-d $tmpdir) {
    mkdir($tmpdir, 0777) || die "can't make temporary directory";
}


##
# Find a writable texmf tree

$writable_texmf = undef;
$_ = `kpsewhich --expand-path='\$TEXMFVAR'`; chomp;
foreach my $dir (split(/:/)) {
    $writable_texmf = $dir, last if -w $dir;
}
if ($writable_texmf && !-f "$writable_texmf/ls-R") {
    system('mktexlsr', $writable_texmf);
}


##
# Read arguments

if ($#ARGV != 0) {
    die "Bad arguments " . join(", ", @ARGV);
}

$want_tfm_name = $ARGV[0];
$want_tfm_name =~ s/\.tfm$//;

$font = new Autofont($want_tfm_name, \%fontmap);

sub create_vpl ($) {
    my($avpl) = @_;
    if (!ref($avpl)) {
	my($afont) = new Autofont($avpl, \%fontmap);
	die "could not make TFM for $avpl" if !generate_tfm($afont);
	$avpl = new Autofont::VPL($afont->{'vpl_file'}, $avpl);
	$avpl->set_encoding($afont->encoding);
    }
    $avpl;
}

my(@digits) = ('/zero', '/one', '/two', '/three', '/four',
	       '/five', '/six', '/seven', '/eight', '/nine');
my(@lowercase) = ('/a', '/b', '/c', '/d', '/e', '/f', '/g', '/h', '/i', '/j',
		  '/k', '/l', '/m', '/n', '/o', '/p', '/q', '/r', '/s', '/t',
		  '/u', '/v', '/w', '/x', '/y', '/z',
		  '/ae', '/oe', '/eth', '/thorn',
		  '/aacute', '/acircumflex', '/adieresis', '/agrave',
		  '/aring', '/atilde',
		  '/ccedilla',
		  '/eacute', '/ecircumflex', '/edieresis', '/egrave',
		  '/iacute', '/icircumflex', '/idieresis', '/igrave',
		  '/lslash',
		  '/ntilde',
		  '/oacute', '/ocircumflex', '/odieresis', '/ograve',
		  '/oslash', '/otilde',
		  '/scaron',
		  '/uacute', '/ucircumflex', '/udieresis', '/ugrave',
		  '/yacute', '/ydieresis', '/zcaron');
my(@uppercase) = ('/A', '/B', '/C', '/D', '/E', '/F', '/G', '/H', '/I', '/J',
		  '/K', '/L', '/M', '/N', '/O', '/P', '/Q', '/R', '/S', '/T',
		  '/U', '/V', '/W', '/X', '/Y', '/Z',
		  '/AE', '/OE', '/Eth', '/Thorn',
		  '/Aacute', '/Acircumflex', '/Adieresis', '/Agrave',
		  '/Aring', '/Atilde',
		  '/Ccedilla',
		  '/Eacute', '/Ecircumflex', '/Edieresis', '/Egrave',
		  '/Iacute', '/Icircumflex', '/Idieresis', '/Igrave',
		  '/Lslash',
		  '/Ntilde',
		  '/Oacute', '/Ocircumflex', '/Odieresis', '/Ograve',
		  '/Oslash', '/Otilde',
		  '/Scaron',
		  '/Uacute', '/Ucircumflex', '/Udieresis', '/Ugrave',
		  '/Yacute', '/Ydieresis', '/Zcaron');
my(@accents) = ('/acute', '/breve', '/caron', '/cedilla', '/circumflex',
		'/dieresis', '/dotaccent', '/grave', '/hungarumlaut',
		'/macron', '/ogonek', '/ring', '/tilde');

sub combine_vpl ($$@) {
    my($avpl, $bvpl, @rest) = @_;
    $avpl = create_vpl($avpl);
    $bvpl = create_vpl($bvpl);
    $avpl->substitute_chars($bvpl, @rest);
    wantarray ? ($avpl, $bvpl) : $avpl;
}


##
# Assign destination directories

$tfm_dest_dir = $ENV{'TFMDESTDIR'};
$vf_dest_dir = $ENV{'VFDESTDIR'};

if ((!$tfm_dest_dir || !$vf_dest_dir) && $writable_texmf) {
    $family = $font->familyname($PSres);
    $family =~ tr/-A-Za-z0-9_.+,//cd if defined($family);
    $family = 'unknown' if !$family;
    $tfm_dest_dir = "$writable_texmf/fonts/tfm/autofont/$family" if !$tfm_dest_dir;
    $vf_dest_dir = "$writable_texmf/fonts/vf/autofont/$family" if !$vf_dest_dir;
}

foreach my $dirref (\$tfm_dest_dir, \$vf_dest_dir) {
    my($dir) = $$dirref;
    if (defined $dir && !-d $dir
	&& substr($dir, 0, length($writable_texmf) + 1) eq "$writable_texmf/"
	&& system("mkdir", "-p", $dir) != 0) {
	$$dirref = undef;
    }
}

die "TFM destination directory not defined\n" if !defined $tfm_dest_dir;
die "VF destination directory not defined\n" if !defined $vf_dest_dir;


##
# Build font

if ($font->{'built'}) {
    my($basename, $suffix) = ($font->{'mmname'}, $font->{'suffix'});
    my($out_vpl) = "$basename$suffix";
    my($c, $d);
    
    if ($font->{'tif'}) {
	my($tifn) = $font->psname($PSres);
	if ($tifn !~ s/SC([^a-z]|\Z)/$1/ && $tifn !~ s/OsF([^a-z]|\Z)/$1/) {
	    die "$tifn is not an SC or OsF font, cannot apply --tif";
	}
	if ($font->ps_fullname_exists("${tifn}TF", $PSres)) {
	    $tifn .= "TF";
	}
	$out_vpl = combine_vpl($out_vpl, $font->replace_base_name($tifn), @digits);
    }
    if ($font->{'osf'}) {
	my($osf_fn) = $font->ps_osf_fullname($PSres);
	die "no OsF font for $basename" if !$osf_fn;
	my(@d);
	foreach $c (@digits) {
	    push @d, [$c, ["${c}oldstyle", $c]];
	}
	$out_vpl = combine_vpl($out_vpl, $osf_fn, @d);
    }
    if ($font->{'rawexpf'}) {
	my($exp_fn) = $font->ps_expert_fullname($PSres);
	$exp_fn =~ s/--E.*?(?=$|--)/--raw/;
	die "no expert font for $basename" if !$exp_fn;
	my(@d);
	for (my $c = 0; $c < 10; $c++) {
	    push @d, [$digits[$c], $c + 48];
	}
	$out_vpl = combine_vpl($out_vpl, $exp_fn, @d);
    }
    if ($font->{'sc'}) {
	my($exp_fn) = $font->ps_expert_fullname($PSres);
	die "no expert font for $basename" if !$exp_fn;
	my(@chars);
	for ($c = 0; $c < @lowercase; $c++) {
	    push @chars, [$lowercase[$c], ["$uppercase[$c]small", $lowercase[$c]]];
	}
	foreach $c (@accents) {
	    push @chars, [$c, [ucfirst($c) . "small", $c]];
	}
	$out_vpl = combine_vpl($out_vpl, $exp_fn, @chars);
	# remove F ligatures
	$out_vpl->remove_lig('/f', '/f');
	$out_vpl->remove_lig('/f', '/i');
	$out_vpl->remove_lig('/f', '/l');
    }
    if ($font->{'swash_caps'}) {
	my($sw_fn) = $font->ps_swash_fullname($PSres);
	die "no swash-caps font for $basename" if !$sw_fn;
	$out_vpl = combine_vpl($out_vpl, $sw_fn, @uppercase);
    }
    if ($font->{'f_ligs'}) {
	my($exp_fn) = $font->ps_expert_fullname($PSres);
	die "no expert font for $basename" if !$exp_fn;
	$out_vpl = combine_vpl($out_vpl, $exp_fn, '/ff', '/ffi', '/ffl');
	$out_vpl->add_lig('/f', '/f', '/ff');
	$out_vpl->add_lig('/ff', '/i', '/ffi');
	$out_vpl->add_lig('/ff', '/l', '/ffl');
	$out_vpl->copy_left_kerns('/ff', '/f');
	$out_vpl->copy_left_kerns('/ffi', '/ff');
	$out_vpl->copy_left_kerns('/ffl', '/ff');
	$out_vpl->copy_right_kerns('/ff', '/f');
	$out_vpl->copy_right_kerns('/ffi', '/i');
	$out_vpl->copy_right_kerns('/ffl', '/l');
    }
    if ($font->{'no_f_ligs'}) {
	$out_vpl = create_vpl($out_vpl);
	$out_vpl->remove_lig('/f', '/f');
	$out_vpl->remove_lig('/f', '/i');
	$out_vpl->remove_lig('/f', '/l');
    }
    if ($font->{'letterspace'}) {
	$out_vpl = create_vpl($out_vpl);
	my($letsp) = $font->{'letterspace'};
	$out_vpl->each_char(sub {
	    s/\(CHARWD R ([\w.]+)\)/"(CHARWD R " . ($1 + $letsp) . ")"/sge;
	});
    }
    if ($font->{'spacestretch'}) {
	$out_vpl = create_vpl($out_vpl);
	$out_vpl->{'FONTDIMEN'} =~ s/STRETCH ([RD]) ([\d.]+)/"STRETCH $1 " . $2*$font->{'spacestretch'}/e;
    }

    die if !ref($out_vpl);
    $font->{'vpl_file'} = "$tmpdir/$want_tfm_name.vpl";
    $out_vpl->write($font->{'vpl_file'});
}



##
# run

if (!$tfm_dest_dir || !$vf_dest_dir || !generate_tfm($font)) {
    die if $font->{'built'};
    # Find another MakeTeXTFM
    my($ok, $p) = 0;
    foreach $p (split /:/, $ENV{'PATH'}) {
	exec "$p/mktextfm $want_tfm_name.tfm" if $ok && -x "$p/mktextfm";
	exec "$p/MakeTeXTFM $want_tfm_name.tfm" if $ok && -x "$p/MakeTeXTFM";
	$ok = 1 if substr($0, 0, length($p) + 1) eq "$p/";
    }
    die "can't find another mktextfm to run";
}


##
# set up psfonts.*

sub add_created_fonts ($) {
    my($psfonts_map) = $_[0];
    my($font);
    foreach $font (@real_fonts) {
	my($t) = $font->psfonts_map_line($PSres);
	if (defined($t)) {
	    $psfonts_map->{'autofonts'}->{ "$font->{'basename'}${Autofont::suffix_separator}0" } = $t;
	    $psfonts_map->{'autofonts'}->{ $font->{'basename'} } = $t
		if $font->{'want_raw'};
	}
    }
}

my $tds_1_1 = (`kpsewhich --expand-path='\$TEXMF/fonts/enc'` =~ /\S/ ? 1 : 0);
my $mapdir = $tds_1_1 ? "$writable_texmf/fonts/map" : "$writable_texmf/dvips";
if ($writable_texmf && (-d $mapdir || system("mkdir", "-p", $mapdir) == 0)) {
    if (!-f "$mapdir/autofont.map") {
	open(MAP, "+>$mapdir/autofont.map") || die;
	system("mktexlsr");
    } else {
	open(MAP, "+<$mapdir/autofont.map") || die;
    }
    flock(MAP, LOCK_EX);

    my(%map);
    while (<MAP>) {
	$map{$1} = $2 if /^([-A-Za-z0-9_.+,]\S*)\s+(.*)/;
    }
    my($created) = (tell(MAP) == 0);

    # add changed map lines for fonts
    my($changed) = 0;
    foreach my $font (@real_fonts) {
	my($t) = $font->psfonts_map_line($PSres);
	$t =~ s|(<\[?)(\S*/)([^\s/]+)|$1$3|g;
	
	my($b) = $font->{'basename'};
	my($f1) = $b . $Autofont::suffix_separator . '0';
	my($f2) = ($font->{'want_raw'} ? $b : undef);
	if (defined($t)) {
	    $map{$f1} = $t, $changed = 1
		if $f1 && (!exists $map{$f1} || $map{$f1} ne $t);
	    $map{$f2} = $t, $changed = 1
		if $f2 && (!exists $map{$f2} || $map{$f2} ne $t);
	}
    }

    if ($changed) {
	seek(MAP, 0, 0);
	print MAP "% Automatically maintained by autofont or other programs. Do not edit.\n\n";
	my($k, $v);
	while (($k, $v) = each %map) {
	    print MAP $k, " ", $v, "\n";
	}
    }

    close(MAP);

    if ($changed && -x "$mapdir/updmap") {
	system("cd $writable_texmf/dvips && ./updmap");
    } elsif ($changed) {
	system("updmap --enable Map autofont.map");
    }
}


##
# end processing

exit if $anyexitvalue;

print "$tfm_dest_dir/$font->{'basename'}.tfm\n";

__END__

=head1 NAME

autofont-mktextfm - create a TFM file for a font (Autofont version)

=head1 SYNOPSIS

autofont-mktextfm I<font>

=head1 DESCRIPTION

B<autofont-mktextfm> generates a TFM file for a given font name. The
Autofont version is designed for PostScript Type 1 fonts. It generates TFM
and VF files automatically from Adobe AFM files. It also modifies
B<dvips>'s F<psfonts.map> configuration files to support any fonts it
creates.

B<autofont-mktextfm> takes the same arguments as L<mktextfm(1)>, and like
that program, it is designed to be called automatically by L<tex(1)> and
related programs. To facilitate this, you should probably install a
symbolic link in your path so that `B<mktextfm>' actually runs
B<autofont-mktextfm>. Leave the original B<mktextfm> later in your path;
B<autofont-mktextfm> will run it for Metafonts and other fonts it cannot
process itself.

B<autofont-mktextfm> depends on PostScript F<PSres.upr> files generated by
L<autofont-mkpsres(1)> or L<makepsres(1)>.

=head1 FONT NAMES

B<autofont-mktextfm> assumes that TeX font names for PostScript fonts equal
the relevant PostScript font names. (It does not adhere to the "fontname"
standard that brought you C<pacrx>.) Thus, the TeX font name
C<Times-Roman> corresponds to the PostScript Times-Roman font.

You can abbreviate font names in the Macintosh style if you prefer. This
style eliminates all dashes from the font name, then preserves at most five
characters from the first "word" and three characters from each succeeding
"word" in the resulting name, where "words" are delimited by capital
letters and/or digits. Thus, B<autofont-mktextfm> will map C<TimesRom> to
Times-Roman, C<ACasReg> to ACaslon-Regular, and C<LetteGot12PitBTRom> to
LetterGothic12PitchBT-Roman.

Autofont can also handle multiple-master fonts. To name a multiple-master
font, append the numbers specifying the desired font instance to the font
name, surrounded and separated by underscores. For example,
C<AJensonMM_454_66_> specifies a version of Adobe Jenson with weight 454
and optical size 66. The final underscore is required.

=head1 FONT INSTRUCTIONS

B<autofont-mktextfm> can parse and execute instructions embedded in a font
name. Each instruction follows two dashes C<-->. For example,
C<Times-Roman--sl16> denotes an artificially slanted version of
Times-Roman, and C<Helvetica--E8r--cn80--fsc> denotes a version of
Helvetica in the 8r encoding, condensed by 80%, and with faked small caps.

Specific instructions are:

=over 8

=item B<--osf>

Use old-style figures taken from a corresponding expert, OsF, or small-caps
font.

=item B<--tif>

Use titling figures taken from a normal font. This was designed for
creating small-caps fonts with titling figures.

=item B<--rawexpf>

Use figures taken from a corresponding expert font (raw encoding).

=item B<--sc>

Use small caps taken from a corresponding expert font.

=item B<--fsc>

Include faked small caps (scaled-down versions of the regular caps).

=item B<--swc>

Use swash caps taken from a corresponding swash or swash-caps font.

=item B<--f>

Include f-ligatures taken from a corresponding expert font.

=item B<--nof>

Do not use any f-ligatures, not even fi or fl.

=item B<--sl>I<FACTOR>

Slant the output by I<FACTOR>. A I<FACTOR> of 167 means slant characters by
one-sixth to the right; 0 is normal.

=item B<--ex>I<PERCENT>

Expand the output by I<PERCENT>. 200 creates a font twice as wide as the
base; 100 is normal; 50 creates a font half as wide.

=item B<--cn>I<PERCENT>

Same as B<--ex>I<PERCENT>.

=item B<--lsp>I<SPACING>

Letterspace the output by I<SPACING>. A I<SPACING> of 20 adds .020 em of
width to each character.

=item B<--spstr>I<PERCENT>

Increase the font's space stretch value by I<PERCENT>. 200 creates a font
with twice as much space stretch as usual.

=item B<--E>I<ENCODING>

Re-encode the font with I<ENCODING>. B<autofont-mktextfm> will search for
an encoding file named F<I<ENCODING>.enc> on the encoding path.

=item B<-->I<ENCODING>

Any unknown instruction is assumed to be an encoding. 

=back

B<autofont-mktextfm> uses some simple heuristics to find the expert font
name corresponding to a normal font name. Please report a bug if these
heuristics fail on some font.

=head1 ENVIRONMENT

B<autofont-mktextfm> writes its output files into a TeX Directory Structure
hierarchy. It first searches your C<$TEXMF> path for the first writable
directory. In many web2c-based installations, this will be your
"C<$HOME/texmf>" directory. Then it places output files in type-specific
specific subdirectories:

 File type    Directory                           Filename
 TFM          TEXMF/fonts/tfm/autofont/TYPEFACE/  WHATEVER.tfm
 VF           TEXMF/fonts/vf/autofont/TYPEFACE/   WHATEVER.vf
 psfonts.map  TEXMF/dvips/                        autofont.map

TEXMF is the writable TeX Directory Structure. TYPEFACE is the font's
family name, or "unknown" if that name cannot be determined.
B<autofont-mktextfm> will also update F<TEXMF/ls-R> as required to
reference any files it created.

Thus, to support B<autofont>, you should do at least the following:

=over 3

=item 1.

Make sure your C<$TEXMF> path contains a writable directory.

=item 2.

Include F<autofont.map> in your L<dvips(1)> configuration files, usually by
including a line like this in F<config.ps>:

 p +autofont.map

=item 3.

Run B<autofont-mkpsres> in the directory or directories that contain your
fonts.

=item 4.

Make sure that B<dvips> can find your fonts, by adding their directory or
directories to your C<T1FONTS> and/or C<TEXPSHEADERS> environment
variables.

=back

=head1 ENVIRONMENT VARIABLES

B<autofont-mktextfm> uses environment variables and the Kpathsea library
(see L<kpsewhich(1)>) to locate resources and to determine where to place
its generated output. In particular:

=over 8

=item F<PSres.upr> files

Searches kpathsea's Type 1 font path (usually T1FONTS).

=item Encoding files (like F<8r.enc>)

Searches kpathsea's TEXPSHEADERS path.

=item Default encoding

Uses TEXDEFAULTENCODING, which defaults to 8r. This encoding is used for
fonts without an explicit encoding specification.

=item Other B<mktextfm> scripts

Searches PATH.

=item TFM output files

Places in TFMDESTDIR, which defaults to
L<TEXMF/fonts/tfm/autofont/TYPEFACE>.

=item VF output files

Places in VFDESTDIR, which defaults to L<TEXMF/fonts/vf/autofont/TYPEFACE>.

=item B<afm2tfm>, B<vptovf>, and B<mmafm> programs

Uses AFM2TFM, VPTOVF, and MMAFM, which default to C<afm2tfm>,
C<vptovf>, and C<mmafm>, respectively.

=head1 BUGS

Report bugs and send patches and enhancement requests to
C<ekohler@gmail.com>.

=head1 SEE ALSO

L<mktextfm(1)>, L<tex(1)>, L<kpsewhich(1)>, L<afm2tfm(1)>, L<mmafm(1)>,
L<dvips(1)>, L<autofont-mkpsres(1)>, L<makepsres(1)>

=head1 AUTHOR

Eddie Kohler, C<ekohler@gmail.com>

See L<http://www.lcdf.org/type/> for updates, and for the mminstance
package, which contains B<mmafm>.

EOD

require Config;
$cfg{'startperl'} = $Config::Config{'startperl'};
$cfg{'libdir'} = `make libdir.txt >/dev/null 2>&1 && cat libdir.txt`;
chomp $cfg{'libdir'};

$script =~ s/\~(\w+)\~/$cfg{$1}/eg;

if (!open(FILE, ">autofont-mktextfm") || !print FILE ($script) || !close(FILE)) {
    die "Error while writing script: $!";
}
