package Autofont::PSres;

my(%directory_prefix) = (
    'FontAFM' => 1, 'FontAMFM' => 1, 'FontOutline' => 1
);

sub new ($@) {
    my($class, @fontpath) = @_;
    my($self) = { };
    bless $self, $class;
    foreach my $elem (reverse @fontpath) {
	$self->read("$elem/PSres.upr") if -r "$elem/PSres.upr";
    }
    $self;
}

sub read ($$;$) {
    my($psres, $fn, $dir) = @_;
    local($_);

    $dir = '' if !defined($dir);
    $dir .= $1 if $fn =~ m|^(.*/)[^/]*$|;
    $dir .= '/' if $dir =~ m|[^/]$|;

    my($section_number, $section, $prefix, $h) = 0;
    open(IDX, $fn) || return undef;
    while (<IDX>) {
	if (m|^/| && $section_number == 1) {
	    last if !defined($_ = <IDX>);
	}
	$section_number++;
	
	chomp($section = $_);
	$prefix = ($directory_prefix{$section} ? $dir : '');
	$psres->{$section} = { } if !exists($psres->{$section});
	$h = $psres->{$section};

	while (<IDX>) {
	    last if /^\.$/;
	    if (/^([^=]+)=(.*)$/) {
		$h->{$1} = $prefix . $2;
	    }
	}
    }
    close IDX;
    
    $psres;
}

sub fontfamily ($$) {
    my($psres, $fontname) = @_;
    return undef if !defined $fontname;

    if (!$psres->{'_fontfamily'}) {
	my($ff) = { };
	my($k, $v, $i);
	while (($k, $v) = each %{$psres->{'FontFamily'}}) {
	    my(@x) = split(/,/, $v);
	    for ($i = 1; $i < @x; $i += 2) {
		$ff->{$x[$i]} = $k;
	    }
	}
	$psres->{'_fontfamily'} = $ff;
    }

    $psres->{'_fontfamily'}->{$fontname};
}

1;
